//----------------------------------------------------------------------------------
// File:   Denoising.cpp
// Author: Alexander Kharlamov
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma warning(disable: 4995)

#include "DXUT.h"
#include "DXUTgui.h"
#include "sdkmisc.h"
#include "DXUTsettingsDlg.h"
#include "../../Include/NVUT/NVUTmedia.h"

#define DEFAULT_SIZE_X            320.0f
#define DEFAULT_SIZE_Y            408.0f

#define MAX_WCHAR_SIZE               260
#define IDC_TOGGLEFULLSCREEN           1
#define IDC_TOGGLEREF                  2
#define IDC_CHANGEDEVICE               3
#define IDC_NOISE_LEVEL_STATIC         4
#define IDC_NOISE_LEVEL_SLIDER         5
#define IDC_LERP_COEFFICIENT_STATIC    6
#define IDC_LERP_COEFFICIENT_SLIDER    7
#define IDC_WEIGHT_THRESHOLD_STATIC    8
#define IDC_WEIGHT_THRESHOLD_SLIDER    9
#define IDC_COUNTER_THRESHOLD_STATIC  10
#define IDC_COUNTER_THRESHOLD_SLIDER  11
#define IDC_GAUSSIAN_SIGMA_STATIC     12
#define IDC_GAUSSIAN_SIGMA_SLIDER     13
#define IDC_WINDOW_RADIUS_STATIC      14
#define IDC_WINDOW_RADIUS_SLIDER      15
#define IDC_BLOCK_RADIUS_STATIC       16
#define IDC_BLOCK_RADIUS_SLIDER       17
#define IDC_MODE_STATIC               18
#define IDC_TECHNIQUES_COMBO          19
#define IDC_DEFAULT_SIZE              20
#define IDC_SAVE                      21
#define IDC_LOAD                      22

struct SVertexTexcoords
{
    D3DXVECTOR3 Pos; 
    D3DXVECTOR2 Tex;
};

int g_SizeX = -1;
int g_SizeY = -1;
int g_CurrentTechnique = 0;

float g_Shift[2] = {0.0f, 0.0f};
float g_TexScale[2] = {1.0f, 1.0f};
float g_TexShift[2] = {0.0f, 0.0f};
float g_NoiseLevel = 0.15f;
float g_WeightThreshold = 0.03f;
float g_LerpCoefficeint = 0.80f;
float g_CounterThreshold = 0.05f;

float g_GaussianSigma = 49.0f;
int   g_WindowRadius = 3;
int   g_BlockRadius = 2;

UINT  g_Stride;
UINT  g_Offset;

bool  g_bShowGUI  = true;
bool  g_bShowHelp = false;
bool  g_bFXInited = false;
bool  g_bSaveToFile = false;

int   g_Pass = 0;

WCHAR g_wcCleanFileName[MAX_WCHAR_SIZE] = L"../../Media/Portrait/Clean.png";
WCHAR g_wcNoisyFileName[MAX_WCHAR_SIZE] = L"../../Media/Portrait/Noisy.png";
WCHAR g_wcSaveFileName[MAX_WCHAR_SIZE];
WCHAR g_wcTitle[MAX_WCHAR_SIZE]            = L"Denoising";
WCHAR g_wcNoiseLevel[MAX_WCHAR_SIZE]       = L"Noise    Level:";
WCHAR g_wcWeightThreshold[MAX_WCHAR_SIZE]  = L"Weight   Threshold:";
WCHAR g_wcLerpCoefficeint[MAX_WCHAR_SIZE]  = L"Lerp     Coefficeint:";
WCHAR g_wcCounterThreshold[MAX_WCHAR_SIZE] = L"Counter  Threshold:";
WCHAR g_wcGaussianSigma[MAX_WCHAR_SIZE]    = L"Gaussian Sigma:";
WCHAR g_wcWindowRadius[MAX_WCHAR_SIZE]     = L"Window   Radius:";
WCHAR g_wcBlockRadius[MAX_WCHAR_SIZE]      = L"Block    Radius:";

ID3D10Device*                              g_pd3dDevice = NULL;
ID3D10Effect*                              g_pEffect = NULL;

ID3D10EffectTechnique*                     g_pTechniqueSimple = NULL;
ID3D10EffectTechnique*                     g_pTechniqueKNN_mod = NULL;
ID3D10EffectTechnique*                     g_pTechniqueNLM_mod = NULL;
ID3D10EffectTechnique*                     g_pCurrentTechnique = NULL;

ID3D10InputLayout*                         g_pVertexLayout = NULL;
ID3D10Buffer*                              g_pVertexBuffer = NULL;
ID3D10Buffer*                              g_pVBClean = NULL;
ID3D10Buffer*                              g_pVBNoisy = NULL;
ID3D10Buffer*                              g_pVBRestored = NULL;

ID3D10Resource*                            g_pResource = NULL;

ID3D10Texture2D*                           g_pTextureNoisy = NULL;
ID3D10Texture2D*                           g_pTextureClean = NULL;
ID3D10Texture2D*                           g_pTextureRestored = NULL;

ID3D10ShaderResourceView*                  g_pTextureRVclean = NULL;
ID3D10ShaderResourceView*                  g_pTextureRVnoisy = NULL;
ID3D10ShaderResourceView*                  g_pTextureRVRestored = NULL;

ID3D10EffectShaderResourceVariable*        g_pNoisyTexture = NULL;
ID3D10EffectShaderResourceVariable*        g_pCleanTexture = NULL;
ID3D10EffectShaderResourceVariable*        g_pRestoredTexture = NULL;

ID3D10RenderTargetView*                    g_pRTVRestored = NULL;

ID3D10EffectScalarVariable*                g_pNoiseLevel = NULL;
ID3D10EffectScalarVariable*                g_pLerpCoefficient = NULL;
ID3D10EffectScalarVariable*                g_pWeightThreshold = NULL;
ID3D10EffectScalarVariable*                g_pCounterThreshold = NULL;
ID3D10EffectScalarVariable*                g_pGaussianSigma = NULL;
ID3D10EffectScalarVariable*                g_pWindowRadius = NULL;
ID3D10EffectScalarVariable*                g_pBlockRadius = NULL;
ID3D10EffectVectorVariable*                g_pShift = NULL;
ID3D10EffectVectorVariable*                g_pTexScale = NULL;
ID3D10EffectVectorVariable*                g_pTexShift = NULL;

ID3DX10Font*                               g_pFont = NULL;
ID3DX10Sprite*                             g_pSprite = NULL;

CDXUTDialog                                g_HUD;                    // dialog for standard controls
CDXUTDialog                                g_GUI;                    // sliders
CDXUTDialog                                g_Caption;                // caption
CDXUTDialogResourceManager                 g_DialogResourceManager;
CD3DSettingsDlg                            g_SettingsDlg;
CDXUTDialog                                g_SaveOpenDlg;

D3D10_VIEWPORT                             g_RenderTargetVP;
D3D10_VIEWPORT                             g_VP;

void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
HRESULT InitTextures(WCHAR *pCleanImage, WCHAR *pNoisyImage);
HRESULT InitSurface(UINT width, UINT height);
void SetTechniqueByIndex(int i);

void AdjustWindow(int w, int h)
{
    RECT ClientRect = { 40, 40, 40 + w, 40 + h };    
    LONG WindowStyle = GetWindowLongA(DXUTGetHWND(), GWL_STYLE);
    AdjustWindowRect( &ClientRect, WindowStyle, FALSE );

    MoveWindow( DXUTGetHWND(), ClientRect.left, ClientRect.top, ClientRect.right - ClientRect.left, ClientRect.bottom - ClientRect.top, true); // make sure that client rect equals texture size 
}
HRESULT InitFX()
{
    HRESULT hr;
    WCHAR path[MAX_PATH] = L"";
    WCHAR *pFileName = L"Denoising.fx";

    if ( !g_bFXInited )
    {
        // Thats the first time since device was created that we call InitFX
        // Create Effect
        V_RETURN(NVUTFindDXSDKMediaFileCch(path, MAX_PATH, pFileName));
        V_RETURN(D3DX10CreateEffectFromFile(path, NULL, NULL, "fx_4_0", D3D10_SHADER_PREFER_FLOW_CONTROL, 0, g_pd3dDevice, NULL, NULL, &g_pEffect, NULL, &hr));    

        // Initialize Effect Variables
        g_pTechniqueSimple   = g_pEffect->GetTechniqueByName( "Simple" );
        g_pTechniqueKNN_mod  = g_pEffect->GetTechniqueByName( "tKNN" );
        g_pTechniqueNLM_mod  = g_pEffect->GetTechniqueByName( "tNLM" ); 

        g_pNoiseLevel = g_pEffect->GetVariableByName("g_NoiseLevel")->AsScalar();
        g_pNoiseLevel->SetFloat( 1.0f / (g_NoiseLevel * g_NoiseLevel) );
        g_pLerpCoefficient = g_pEffect->GetVariableByName("g_LerpCoefficient")->AsScalar();
        g_pLerpCoefficient->SetFloat(g_LerpCoefficeint);
        g_pWeightThreshold = g_pEffect->GetVariableByName("g_WeightThreshold")->AsScalar();
        g_pWeightThreshold->SetFloat(g_WeightThreshold);
        g_pCounterThreshold = g_pEffect->GetVariableByName("g_CounterThreshold")->AsScalar();
        g_pCounterThreshold->SetFloat(g_CounterThreshold);
        g_pGaussianSigma = g_pEffect->GetVariableByName("g_GaussianSigma")->AsScalar();
        g_pGaussianSigma->SetFloat( 1.0f / g_GaussianSigma);
        g_pWindowRadius = g_pEffect->GetVariableByName("g_WindowRadius")->AsScalar();
        g_pWindowRadius->SetFloat((float)g_WindowRadius);
        g_pBlockRadius = g_pEffect->GetVariableByName("g_BlockRadius")->AsScalar();
        g_pBlockRadius->SetFloat((float)g_BlockRadius);
        g_pShift = g_pEffect->GetVariableByName("g_Shift")->AsVector();
        g_pTexScale = g_pEffect->GetVariableByName("g_TexScale")->AsVector();
        g_pTexShift = g_pEffect->GetVariableByName("g_TexShift")->AsVector();

        g_pNoisyTexture = g_pEffect->GetVariableByName( "txNoisy" )->AsShaderResource();
        g_pCleanTexture = g_pEffect->GetVariableByName( "txClean" )->AsShaderResource();
        g_pRestoredTexture = g_pEffect->GetVariableByName( "txRestored" )->AsShaderResource();

        g_bFXInited = true;
    }
    
    InitTextures(g_wcCleanFileName, g_wcNoisyFileName);
    InitSurface( (UINT) DEFAULT_SIZE_X, (UINT) DEFAULT_SIZE_Y );

    g_pCleanTexture->SetResource( g_pTextureRVclean );
    g_pNoisyTexture->SetResource( g_pTextureRVnoisy );

    g_Shift[0] = 1.0f / (float) g_SizeX;
    g_Shift[1] = 1.0f / (float) g_SizeY;

    g_TexScale[0] = DEFAULT_SIZE_X / (float) g_SizeX; // default picture is 320 by 408
    g_TexScale[1] = DEFAULT_SIZE_Y / (float) g_SizeY; // a piece of image of this size is shown on the screen

    g_TexShift[0] = 0.0f;
    g_TexShift[1] = 0.0f;

    g_pShift->SetFloatVector( &g_Shift[0] );
    g_pTexScale->SetFloatVector( &g_TexScale[0] );
    g_pTexShift->SetFloatVector( &g_TexShift[0] );

    return S_OK;
}
HRESULT InitTextures(WCHAR *pCleanImage, WCHAR *pNoisyImage)
{
    HRESULT hr;
    D3D10_SHADER_RESOURCE_VIEW_DESC SRVdesc;
    D3D10_TEXTURE2D_DESC desc;
    WCHAR path[MAX_PATH];
   
    SAFE_RELEASE( g_pTextureClean );
    SAFE_RELEASE( g_pTextureNoisy );
    
    V_RETURN(NVUTFindDXSDKMediaFileCch(path, MAX_PATH, pNoisyImage));
    V_RETURN(D3DX10CreateTextureFromFile( g_pd3dDevice, path, NULL, NULL, &g_pResource, &hr ));
    g_pResource->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&g_pTextureNoisy );
    g_pResource->Release();

    V_RETURN(NVUTFindDXSDKMediaFileCch(path, MAX_PATH, pCleanImage));
    V_RETURN(D3DX10CreateTextureFromFile( g_pd3dDevice, path, NULL, NULL, &g_pResource, &hr ));
    g_pResource->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&g_pTextureClean );
    g_pResource->Release();

    SAFE_RELEASE( g_pTextureRVnoisy );
    SAFE_RELEASE( g_pTextureRVclean );

    g_pTextureNoisy->GetDesc( &desc );
    ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
    SRVdesc.Format = desc.Format;
    SRVdesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = desc.MipLevels;

    V_RETURN(g_pd3dDevice->CreateShaderResourceView( g_pTextureClean, &SRVdesc, &g_pTextureRVclean ));
    V_RETURN(g_pd3dDevice->CreateShaderResourceView( g_pTextureNoisy, &SRVdesc, &g_pTextureRVnoisy ));
    
    g_SizeX = desc.Width;
    g_SizeY = desc.Height;

    return S_OK;
}
HRESULT InitSurface(UINT width, UINT height)
{
    HRESULT hr;
    D3D10_SHADER_RESOURCE_VIEW_DESC    SRVdesc;    
    DXGI_SAMPLE_DESC sample_desc = {1, 0};
    D3D10_TEXTURE2D_DESC surface_desc = { width, height, 1, 1, DXGI_FORMAT_R16G16B16A16_FLOAT, sample_desc,
                                          D3D10_USAGE_DEFAULT, D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE, 0, 0};
    SAFE_RELEASE( g_pTextureRestored );
    V_RETURN( g_pd3dDevice->CreateTexture2D( &surface_desc, NULL, &g_pTextureRestored ) );
    
    ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
    SRVdesc.Format = surface_desc.Format;
    SRVdesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = surface_desc.MipLevels;
    
    SAFE_RELEASE( g_pTextureRVRestored );    
    V_RETURN( g_pd3dDevice->CreateShaderResourceView( g_pTextureRestored, &SRVdesc, &g_pTextureRVRestored ) );
    g_pRestoredTexture->SetResource( g_pTextureRVRestored );
    
    SAFE_RELEASE( g_pRTVRestored );
    V_RETURN( g_pd3dDevice->CreateRenderTargetView( (ID3D10Resource *) g_pTextureRestored, NULL, &g_pRTVRestored ) );

    return S_OK;
}
HRESULT InitDialog()
{
    int iY = 10;

    g_SettingsDlg.Init( &g_DialogResourceManager );
    g_GUI.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );
    g_Caption.Init( &g_DialogResourceManager );
    g_SaveOpenDlg.Init( &g_DialogResourceManager );

    g_Caption.AddStatic( IDC_MODE_STATIC, L"Original Image",      100, 480, 125, 22 );
    g_Caption.AddStatic( IDC_MODE_STATIC, L"Noisy Image",         430, 480, 125, 22 );
    g_Caption.AddStatic( IDC_MODE_STATIC, L"Reconstructed Image", 780, 480, 125, 22 );

    g_HUD.SetCallback( OnGUIEvent ); 
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF,        L"Toggle REF (F3)",    35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE,     L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );
    g_HUD.AddButton( IDC_DEFAULT_SIZE,     L"Default size (F4)",  35, iY += 24, 125, 22, VK_F4 );

    iY = 10;
    g_SaveOpenDlg.SetCallback( OnGUIEvent ); 
    g_SaveOpenDlg.AddButton( IDC_LOAD, L"Load (L)", 100, iY, 125, 22 );
    g_SaveOpenDlg.AddButton( IDC_SAVE, L"Save (S)", 100, iY += 24, 125, 22, VK_F3 );

    CDXUTComboBox* pComboBox = NULL;

    g_GUI.SetCallback( OnGUIEvent );   
    g_GUI.AddStatic( IDC_MODE_STATIC, L"Render mode", 50, 0, 180, 25 );
    g_GUI.AddComboBox( IDC_TECHNIQUES_COMBO, 50, 20, 180, 25, ' ', false, &pComboBox );

    if( pComboBox )
    {
        pComboBox->SetDropHeight( 50 );
        pComboBox->SetHotkey( (UINT) VK_DOWN );
    }

    const int nTechs = 2;
    const WCHAR* RENDER_TECHNIQUES[nTechs] =
    {
        L"KNN output image",
        L"NLM output image",
    };

    for( int i = 0; i < nTechs; ++i )
        pComboBox->AddItem( RENDER_TECHNIQUES[i], IntToPtr(i) );
    
    g_GUI.AddStatic( IDC_NOISE_LEVEL_STATIC, g_wcNoiseLevel, 50, 60, 200, 25); 
    g_GUI.AddSlider( IDC_NOISE_LEVEL_SLIDER, 50, 80, 200, 25, 1, 100, (int) (g_NoiseLevel * 100) );

    g_GUI.AddStatic( IDC_LERP_COEFFICIENT_STATIC, g_wcLerpCoefficeint, 50, 110, 200, 25); 
    g_GUI.AddSlider( IDC_LERP_COEFFICIENT_SLIDER, 50, 130, 200, 25, 0, 100, (int) (g_LerpCoefficeint * 100) );

    g_GUI.AddStatic( IDC_WEIGHT_THRESHOLD_STATIC, g_wcWeightThreshold, 50, 160, 200, 25); 
    g_GUI.AddSlider( IDC_WEIGHT_THRESHOLD_SLIDER, 50, 180, 200, 25, 0, 100, (int) (g_WeightThreshold * 100) );

    g_GUI.AddStatic( IDC_COUNTER_THRESHOLD_STATIC, g_wcCounterThreshold, 50, 210, 200, 25); 
    g_GUI.AddSlider( IDC_COUNTER_THRESHOLD_SLIDER, 50, 230, 200, 25, 0, 100, (int) (g_CounterThreshold * 100) );

    g_GUI.AddStatic( IDC_GAUSSIAN_SIGMA_STATIC, g_wcGaussianSigma, 50, 260, 200, 25);
    g_GUI.AddSlider( IDC_GAUSSIAN_SIGMA_SLIDER, 50, 280, 200, 25, 1, 100, (int) (g_GaussianSigma) );

    g_GUI.AddStatic( IDC_WINDOW_RADIUS_STATIC, g_wcWindowRadius, 50, 310, 200, 25);
    g_GUI.AddSlider( IDC_WINDOW_RADIUS_SLIDER, 50, 330, 200, 25, 1, 10, (int) (g_WindowRadius) );

    g_GUI.AddStatic( IDC_BLOCK_RADIUS_STATIC, g_wcBlockRadius, 50, 360, 200, 25);
    g_GUI.AddSlider( IDC_BLOCK_RADIUS_SLIDER, 50, 380, 200, 25, 1, 10, (int) (g_BlockRadius) );

    return true;
}


HRESULT InitResources(ID3D10Device* pd3dDevice)
{
    HRESULT hr;

    InitFX();

	// Set DEFAULT technique to "Non Local Means"
    CDXUTComboBox* pComboBox = g_GUI.GetComboBox(IDC_TECHNIQUES_COMBO);
    pComboBox->SetSelectedByIndex(1);
    SetTechniqueByIndex(1); 

    // 2 viewports
	// i) for rendering a fullscreen quad
    g_RenderTargetVP.Width  = (UINT) DEFAULT_SIZE_X;
    g_RenderTargetVP.Height = (UINT) DEFAULT_SIZE_Y;
    g_RenderTargetVP.MinDepth = 0.0f;
    g_RenderTargetVP.MaxDepth = 1.0f;
    g_RenderTargetVP.TopLeftX = 0;
    g_RenderTargetVP.TopLeftY = 0;

	// ii) for rendering 3 images and GUI
    RECT rect = DXUTGetWindowClientRect();
    g_VP.Width = rect.right - rect.left;
    g_VP.Height = rect.bottom - rect.top;
    g_VP.MinDepth = 0.0f;
    g_VP.MaxDepth = 1.0f;
    g_VP.TopLeftX = 0;
    g_VP.TopLeftY = 0;
    pd3dDevice->RSSetViewports( 1, &g_RenderTargetVP );

    SAFE_RELEASE( g_pVBClean );
    SAFE_RELEASE( g_pVBNoisy );
    SAFE_RELEASE( g_pVBRestored );
    SAFE_RELEASE( g_pVertexBuffer );
    SAFE_RELEASE( g_pVertexLayout );

    D3D10_INPUT_ELEMENT_DESC layout[] =
    {
      { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },  
      { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 }, 
    };
    UINT numElements = sizeof(layout)/sizeof(layout[0]);

    D3D10_PASS_DESC PassDesc;
    g_pTechniqueSimple->GetPassByName( "Clean" )->GetDesc( &PassDesc );
    
    V_RETURN( pd3dDevice->CreateInputLayout( layout, numElements, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &g_pVertexLayout ) );

    pd3dDevice->IASetInputLayout( g_pVertexLayout );

	// This is for rendering a fullscreen quad 
    SVertexTexcoords vertices[6] =
    {
      { D3DXVECTOR3(-1.0f,  1.0f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 1.0f, -1.0f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
      { D3DXVECTOR3(-1.0f, -1.0f, 0.0f ), D3DXVECTOR2( 0.0f, 1.0f) },

      { D3DXVECTOR3(-1.0f,  1.0f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 1.0f,  1.0f, 0.0f ), D3DXVECTOR2( 1.0f, 0.0f) },
      { D3DXVECTOR3( 1.0f, -1.0f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
    };

    // Assume default window width is 1200 heigth is 540;
	// So: There are 3 images: Clean, Noisy and Restored that we would like to see on the screen
	// Default image is 320x408 
	// These vertex arrays are filled so that 3 images and GUI can be placed neatly on the screen 
    SVertexTexcoords verticesRestored[6] =
    {
      { D3DXVECTOR3( 0.1125f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 0.6425f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
      { D3DXVECTOR3( 0.1125f, -0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 1.0f) },

      { D3DXVECTOR3( 0.1125f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 0.6425f,  0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 0.0f) },
      { D3DXVECTOR3( 0.6425f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
    };
    SVertexTexcoords verticesClean[6] =
    {
      { D3DXVECTOR3(-0.9825f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3(-0.4525f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
      { D3DXVECTOR3(-0.9825f, -0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 1.0f) },

      { D3DXVECTOR3(-0.9825f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3(-0.4525f,  0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 0.0f) },
      { D3DXVECTOR3(-0.4525f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
    };
    SVertexTexcoords verticesNoisy[6] =
    {
      { D3DXVECTOR3(-0.4355f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 0.0955f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
      { D3DXVECTOR3(-0.4355f, -0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 1.0f) },

      { D3DXVECTOR3(-0.4355f,  0.75f, 0.0f ), D3DXVECTOR2( 0.0f, 0.0f) },
      { D3DXVECTOR3( 0.0955f,  0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 0.0f) },
      { D3DXVECTOR3( 0.0955f, -0.75f, 0.0f ), D3DXVECTOR2( 1.0f, 1.0f) },
    };
    D3D10_BUFFER_DESC bd;
    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( SVertexTexcoords ) * 6;
    bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;
    D3D10_SUBRESOURCE_DATA InitData;
    InitData.pSysMem = vertices;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVertexBuffer ) );

    InitData.pSysMem = verticesRestored;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVBRestored ) );

    InitData.pSysMem = verticesClean;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVBClean ) );

    InitData.pSysMem = verticesNoisy;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVBNoisy ) );

    g_Stride = sizeof( SVertexTexcoords );
    g_Offset = 0;
    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVertexBuffer, &g_Stride, &g_Offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    return S_OK;
}

void LoadImage()
{                
    OPENFILENAME info;
    ZeroMemory(&info, sizeof (OPENFILENAME));
    info.lStructSize = sizeof (OPENFILENAME);
    info.hwndOwner = DXUTGetHWND();
    info.lpstrFilter = L"Image formats\0*.PNG;*.BMP;*.JPG;*.DDS\0\0";
    info.nFilterIndex = 1;               
    info.lpstrFile = &g_wcCleanFileName[0];
    info.nMaxFile = MAX_WCHAR_SIZE;
    info.lpstrTitle = L"Open Clean Image";
    info.nMaxFileTitle = sizeof(info.lpstrTitle);
    info.dwReserved = OFN_EXPLORER;
    info.Flags = OFN_NOCHANGEDIR;

    BOOL bResult = FALSE;
    bResult = GetOpenFileName(&info);
    if (!bResult) return;

    info.lpstrFile = &g_wcNoisyFileName[0];
    info.nMaxFile = MAX_WCHAR_SIZE;
    info.lpstrTitle = L"Open Noisy Image";
    info.nMaxFileTitle = sizeof(info.lpstrTitle);
    info.dwReserved = OFN_EXPLORER;

    bResult = FALSE;
    bResult = GetOpenFileName(&info);
    if (!bResult) return;

    InitResources(g_pd3dDevice);
}

void SaveImage()
{
    OPENFILENAME info;
    ZeroMemory(&info, sizeof (OPENFILENAME));
    info.lStructSize = sizeof (OPENFILENAME);
    info.hwndOwner = DXUTGetHWND();               
    info.lpstrFilter = L"DDS\0*.DDS\0\0";
    info.nFilterIndex = 1;               
    info.lpstrFile = &g_wcSaveFileName[0];
    info.nMaxFile = MAX_WCHAR_SIZE;
    info.lpstrTitle = L"Save Restored Image";
    info.nMaxFileTitle = sizeof(info.lpstrTitle);
    info.dwReserved = OFN_EXPLORER;
    info.Flags = OFN_NOCHANGEDIR;

    BOOL bResult = FALSE;
    bResult = GetSaveFileName(&info);
    if (!bResult) return; else g_bSaveToFile = true;
}

// Render text
void RenderText()
{
    const int iHelp = 9;
    const WCHAR* wcHelp[iHelp] =
    {
        L"Press F1 for help",
        L"Recommended screen resolution is 1280x960",
        L"Press '1' to view image restored by SUSAN",
        L"Press '2' to view image restored by NLM",
        L"Press F5 to resize window to default size",
        L"Press Space Bar to show smooth (RED) and edgy (BLUE) areas", 
        L"Press 'L' to load user defined picture. Use right mouse button to scroll",
        L"Press 'S' to save restored image",
        L"Press F1 to hide help",
    };
    CDXUTTextHelper txtHelper(g_pFont, g_pSprite, 15);
    txtHelper.Begin();
    txtHelper.SetInsertionPos( 5, 5 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats(true) );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );
    if (g_bShowHelp)
    {
        for (int i = 1; i < iHelp; i++)
            txtHelper.DrawTextLine( wcHelp[i] );        
    }
    else
    {
        txtHelper.DrawTextLine( wcHelp[0] );
    }

    txtHelper.End();
}

void EnableDenoisingGUI(bool enabled)
{
    g_GUI.GetStatic(IDC_NOISE_LEVEL_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_LERP_COEFFICIENT_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_WEIGHT_THRESHOLD_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_COUNTER_THRESHOLD_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_GAUSSIAN_SIGMA_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_WINDOW_RADIUS_STATIC)->SetEnabled(enabled);
    g_GUI.GetStatic(IDC_BLOCK_RADIUS_STATIC)->SetEnabled(enabled);

    g_GUI.GetSlider(IDC_NOISE_LEVEL_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_LERP_COEFFICIENT_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_WEIGHT_THRESHOLD_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_COUNTER_THRESHOLD_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_GAUSSIAN_SIGMA_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_WINDOW_RADIUS_SLIDER)->SetEnabled(enabled);
    g_GUI.GetSlider(IDC_BLOCK_RADIUS_SLIDER)->SetEnabled(enabled);
}

// All changes to the technique are directed through here to ensure GUI consistency.
void SetTechniqueByIndex(int i)
{
    switch (i)
    {
        case 0: g_pCurrentTechnique = g_pTechniqueKNN_mod;
          EnableDenoisingGUI(true);
          g_GUI.GetSlider(IDC_BLOCK_RADIUS_SLIDER)->SetEnabled(false); // block radius doesnt change anything in KNN
        break;        
        case 1: g_pCurrentTechnique = g_pTechniqueNLM_mod;
          EnableDenoisingGUI(true);
        break;
    }
}

void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
    switch( nControlID )
    {
      case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
      case IDC_TOGGLEREF:        DXUTToggleREF(); break;
      case IDC_CHANGEDEVICE:     g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); break;
      case IDC_DEFAULT_SIZE:     if (DXUTIsWindowed()) AdjustWindow(1200, 540);
      case IDC_NOISE_LEVEL_SLIDER:
        g_NoiseLevel = (float) (g_GUI.GetSlider( IDC_NOISE_LEVEL_SLIDER )->GetValue() * 0.01f);
        g_pNoiseLevel->SetFloat( 1.0f / (g_NoiseLevel * g_NoiseLevel) );
        swprintf(&g_wcNoiseLevel[15], MAX_WCHAR_SIZE, L"  %5.2f", g_NoiseLevel);
        g_GUI.GetStatic( IDC_NOISE_LEVEL_STATIC )->SetText(g_wcNoiseLevel);
        break;
      case IDC_LERP_COEFFICIENT_SLIDER:
        g_LerpCoefficeint = (float) (g_GUI.GetSlider( IDC_LERP_COEFFICIENT_SLIDER )->GetValue() * 0.01f);
        g_pLerpCoefficient->SetFloat( g_LerpCoefficeint );
        swprintf(&g_wcLerpCoefficeint[21], MAX_WCHAR_SIZE, L"  %5.2f", g_LerpCoefficeint);
        g_GUI.GetStatic( IDC_LERP_COEFFICIENT_STATIC )->SetText(g_wcLerpCoefficeint);
        break;
      case IDC_WEIGHT_THRESHOLD_SLIDER:
        g_WeightThreshold = (float) (g_GUI.GetSlider( IDC_WEIGHT_THRESHOLD_SLIDER )->GetValue() * 0.01f);
        g_pWeightThreshold->SetFloat( g_WeightThreshold );
        swprintf(&g_wcWeightThreshold[19], MAX_WCHAR_SIZE, L"  %5.2f", g_WeightThreshold);
        g_GUI.GetStatic( IDC_WEIGHT_THRESHOLD_STATIC )->SetText(g_wcWeightThreshold);
        break;
      case IDC_COUNTER_THRESHOLD_SLIDER:
        g_CounterThreshold = (float) (g_GUI.GetSlider( IDC_COUNTER_THRESHOLD_SLIDER )->GetValue() * 0.01f);
        g_pCounterThreshold->SetFloat( g_CounterThreshold );
        swprintf(&g_wcCounterThreshold[19], MAX_WCHAR_SIZE, L"  %5.2f", g_CounterThreshold);
        g_GUI.GetStatic( IDC_COUNTER_THRESHOLD_STATIC )->SetText(g_wcCounterThreshold);
        break;
      case IDC_GAUSSIAN_SIGMA_SLIDER:
        g_GaussianSigma = (float) (g_GUI.GetSlider( IDC_GAUSSIAN_SIGMA_SLIDER )->GetValue() * 1.0f);
        g_pGaussianSigma->SetFloat( 1.0f / g_GaussianSigma );
        swprintf(&g_wcGaussianSigma[15], MAX_WCHAR_SIZE, L"  %5.2f", g_GaussianSigma);
        g_GUI.GetStatic( IDC_GAUSSIAN_SIGMA_STATIC )->SetText(g_wcGaussianSigma);
        break;
      case IDC_WINDOW_RADIUS_SLIDER:
        g_WindowRadius = g_GUI.GetSlider( IDC_WINDOW_RADIUS_SLIDER )->GetValue();
        g_pWindowRadius->SetFloat( (float) g_WindowRadius );
        swprintf(&g_wcWindowRadius[15], MAX_WCHAR_SIZE, L"  %d", g_WindowRadius);
        g_GUI.GetStatic( IDC_WINDOW_RADIUS_STATIC )->SetText(g_wcWindowRadius);
        break;
      case IDC_BLOCK_RADIUS_SLIDER:
        g_BlockRadius = g_GUI.GetSlider( IDC_BLOCK_RADIUS_SLIDER )->GetValue();
        g_pBlockRadius->SetFloat( (float) g_BlockRadius );
        swprintf(&g_wcBlockRadius[15], MAX_WCHAR_SIZE, L"  %d", g_BlockRadius);
        g_GUI.GetStatic( IDC_BLOCK_RADIUS_STATIC )->SetText(g_wcBlockRadius);
        break;
      case IDC_TECHNIQUES_COMBO:
          {
            CDXUTComboBox* pComboBox = g_GUI.GetComboBox(IDC_TECHNIQUES_COMBO);
            const int nTechnique = pComboBox->GetSelectedIndex();
            SetTechniqueByIndex(nTechnique);
            break;
          }
      case IDC_SAVE: SaveImage(); break;
      case IDC_LOAD: LoadImage(); break;
    } 
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;
    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, L"Arial", &g_pFont ) );

    g_pd3dDevice = pd3dDevice;

    hr = InitResources(g_pd3dDevice);

    return S_OK;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;
    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_SettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

    g_GUI.SetLocation( pBackBufferSurfaceDesc->Width - 260, 110);
    g_GUI.SetSize( 150, 150 );
    
    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 220, 0);
    g_HUD.SetSize( 170, 170 );

    g_Caption.SetLocation( 0, pBackBufferSurfaceDesc->Height - 540);
    g_Caption.SetSize( pBackBufferSurfaceDesc->Width , 25 );
    g_Caption.RemoveAllControls();
    g_Caption.AddStatic( IDC_MODE_STATIC, L"Original Image",      10, 480, 125, 22 );
    g_Caption.AddStatic( IDC_MODE_STATIC, L"Noisy Image",         int(pBackBufferSurfaceDesc->Width/3.5), 480, 125, 22 );
    g_Caption.AddStatic( IDC_MODE_STATIC, L"Reconstructed Image", int(pBackBufferSurfaceDesc->Width/1.7), 480, 125, 22 );

    g_SaveOpenDlg.SetLocation( pBackBufferSurfaceDesc->Width - 410, 0);
    g_SaveOpenDlg.SetSize( 150, 150 );

    g_VP.Height = pBackBufferSurfaceDesc->Height;
    g_VP.Width  = pBackBufferSurfaceDesc->Width;

    return S_OK;
}

//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
}

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
    pd3dDevice->ClearRenderTargetView( DXUTGetD3D10RenderTargetView(), ClearColor );
    pd3dDevice->ClearDepthStencilView( DXUTGetD3D10DepthStencilView(), D3D10_CLEAR_DEPTH, 1.0, 0 );

    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    ID3D10RenderTargetView* pOldRTV;
    ID3D10DepthStencilView* pOldDSV;
    ID3D10RenderTargetView* pMultipleRT[] = { g_pRTVRestored };

	// Saving filtered image into a file
    if (g_bSaveToFile)
    {     
		// i) Make a texture of appropriete size
        D3D10_VIEWPORT vp = { 0, 0, g_SizeX, g_SizeY, 0, 1};
        D3D10_SHADER_RESOURCE_VIEW_DESC SRVdesc;    
        DXGI_SAMPLE_DESC sample_desc = {1, 0};
        D3D10_TEXTURE2D_DESC surface_desc = { g_SizeX, g_SizeY, 1, 1, DXGI_FORMAT_R16G16B16A16_FLOAT, sample_desc,
                                              D3D10_USAGE_DEFAULT, D3D10_BIND_RENDER_TARGET, 0, 0};
        ID3D10Texture2D* pTextureSaveToFile = NULL;
        pd3dDevice->CreateTexture2D( &surface_desc, NULL, &pTextureSaveToFile );

        ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
        SRVdesc.Format = surface_desc.Format;
        SRVdesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        SRVdesc.Texture2D.MipLevels = surface_desc.MipLevels;

        ID3D10ShaderResourceView* pTextureRVSaveToFile = NULL;   
        pd3dDevice->CreateShaderResourceView( pTextureSaveToFile, &SRVdesc, &pTextureRVSaveToFile );        

        ID3D10RenderTargetView* pRTVSaveToFile = NULL;
        pd3dDevice->CreateRenderTargetView( (ID3D10Resource *) pTextureSaveToFile, NULL, &pRTVSaveToFile );

		// ii) Render image into it
        pd3dDevice->OMGetRenderTargets(1, &pOldRTV, &pOldDSV);
        pd3dDevice->OMSetRenderTargets(1, &pRTVSaveToFile, NULL);
        pd3dDevice->RSSetViewports(1, &vp);

        pd3dDevice->IASetInputLayout( g_pVertexLayout );
        pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVertexBuffer, &g_Stride, &g_Offset );
        pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
        g_pCurrentTechnique->GetPassByIndex( g_Pass + 2 )->Apply( 0 ); // shift pass to use VSRestored() VSRestored doesnt scale or shift texture coordinates
        pd3dDevice->Draw( 6, 0 );

        pd3dDevice->OMSetRenderTargets(1, &pOldRTV, pOldDSV);
        SAFE_RELEASE( pOldRTV );
        SAFE_RELEASE( pOldDSV ); 

		// iii) And then save
        D3DX10SaveTextureToFile(pTextureSaveToFile, D3DX10_IFF_DDS, g_wcSaveFileName);       

        SAFE_RELEASE( pTextureSaveToFile );
        SAFE_RELEASE( pTextureRVSaveToFile );
        SAFE_RELEASE( pRTVSaveToFile );

        g_bSaveToFile = false;
    }

	// Every frame only a block of image is filtered. 
	// The result is rendered into an RGBA_FLOAT16 texture
    pd3dDevice->OMGetRenderTargets(1, &pOldRTV, &pOldDSV);
    pd3dDevice->OMSetRenderTargets(1, &pMultipleRT[0], NULL);
    pd3dDevice->RSSetViewports(1, &g_RenderTargetVP);

    pd3dDevice->IASetInputLayout( g_pVertexLayout );
    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVertexBuffer, &g_Stride, &g_Offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
    g_pCurrentTechnique->GetPassByIndex( g_Pass )->Apply( 0 );
    pd3dDevice->Draw( 6, 0 );

	// Now restore old render target and draw 3 quads with clean, noisy and restored image
    pd3dDevice->OMSetRenderTargets(1, &pOldRTV, pOldDSV);
    pd3dDevice->RSSetViewports(1, &g_VP);

    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVBClean, &g_Stride, &g_Offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
    g_pTechniqueSimple->GetPassByName( "Clean" )->Apply(0);
    pd3dDevice->Draw( 6, 0 );

    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVBNoisy, &g_Stride, &g_Offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
    g_pTechniqueSimple->GetPassByName( "Noisy" )->Apply(0);
    pd3dDevice->Draw( 6, 0 );

    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVBRestored, &g_Stride, &g_Offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
    g_pTechniqueSimple->GetPassByName( "Restored" )->Apply(0);
    pd3dDevice->Draw( 6, 0 );

    SAFE_RELEASE( pOldRTV );
    SAFE_RELEASE( pOldDSV );   

    g_GUI.OnRender( fElapsedTime );
    g_HUD.OnRender( fElapsedTime );
    g_Caption.OnRender( fElapsedTime );
    g_SaveOpenDlg.OnRender( fElapsedTime );

    RenderText();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_SettingsDlg.OnD3D10DestroyDevice();

    if( g_pd3dDevice ) g_pd3dDevice->ClearState();
    SAFE_RELEASE( g_pVertexBuffer );
    SAFE_RELEASE( g_pVBClean );
    SAFE_RELEASE( g_pVBNoisy );
    SAFE_RELEASE( g_pVBRestored );
    SAFE_RELEASE( g_pVertexLayout );
    SAFE_RELEASE( g_pEffect );
    SAFE_RELEASE( g_pTextureClean );
    SAFE_RELEASE( g_pTextureNoisy );
    SAFE_RELEASE( g_pTextureRestored );
    SAFE_RELEASE( g_pTextureRVnoisy );
    SAFE_RELEASE( g_pTextureRVclean );
    SAFE_RELEASE( g_pTextureRVRestored );
    SAFE_RELEASE( g_pRTVRestored );
    SAFE_RELEASE( g_pFont );    

    g_bFXInited = false; // This variable controls shader init.
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );

    if( *pbNoFurtherProcessing )
        return 0;

    // Pass messages to settings dialog if its active
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the sliders and combobox a chance to handle the message first
    *pbNoFurtherProcessing = g_GUI.MsgProc( hWnd, uMsg, wParam, lParam );

    // Give the buttons a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );

    // Give the Save/Load a chance to handle the message first
    *pbNoFurtherProcessing = g_SaveOpenDlg.MsgProc( hWnd, uMsg, wParam, lParam );

    if( *pbNoFurtherProcessing )
        return 0;

    return 0;
}

//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( !bKeyDown )
        return;
    switch (nChar)
    {
        case '1':
        case '2':
        {
            const int nTechnique = nChar - '1';
            SetTechniqueByIndex(nTechnique);
            CDXUTComboBox* pComboBox = g_GUI.GetComboBox(IDC_TECHNIQUES_COMBO);
            pComboBox->SetSelectedByIndex(nTechnique);
        }
        break;
        case VK_F4: if (DXUTIsWindowed()) AdjustWindow(1200, 540); break;
        case VK_F3: DXUTToggleREF(); break;
        case VK_F1: g_bShowHelp = !g_bShowHelp; break;
        case ' ': g_Pass = !g_Pass; break; // toggle to "Show areas" mode. RED - smooth areas, BLUE - edgy areas
        case 'L': LoadImage(); break;
        case 'S': SaveImage(); break;
    }
}

//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                       bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                       int xPos, int yPos, void* pUserContext )
{
    static int iPosX;
    static int iPosY;  

	// This is a picture scrolling code
    if (bRightButtonDown)
    {		
        int dx = iPosX - xPos;
        int dy = iPosY - yPos;

        float fShift[2];

        fShift[0] = dx * 0.0005f; 
        fShift[1] = dy * 0.0005f;

        if ((g_TexShift[0] + g_TexScale[0] + fShift[0] <= 1.0f) && (g_TexShift[0] + fShift[0] >= 0.0f) && 
            (g_TexShift[1] + g_TexScale[1] + fShift[1] <= 1.0f) && (g_TexShift[1] + fShift[1] >= 0.0f) )
        {
            g_TexShift[0] += fShift[0];
            g_TexShift[1] += fShift[1];
        }

        g_pTexShift->SetFloatVector( &g_TexShift[0] ); // the actual shifting and texture coordinate scaling is done in the vertex shader
    }
    else
    {
        iPosX = xPos;
        iPosY = yPos;
    }
}

//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove      ( OnFrameMove );
    DXUTSetCallbackKeyboard       ( OnKeyboard );
    DXUTSetCallbackMouse          ( OnMouse, TRUE );
    DXUTSetCallbackMsgProc        ( MsgProc );
    DXUTSetCallbackDeviceChanging ( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved  ( OnDeviceRemoved );

    // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable   ( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated      ( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized   ( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender        ( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing ( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed    ( OnD3D10DestroyDevice );

    // IMPORTANT: set SDK media search path to include source directory of this sample, when started from .\Bin
    HRESULT hr;
    V_RETURN(DXUTSetMediaSearchPath(L"..\\Source\\Denoising"));
    
    DXUTInit( true, true, NULL );        // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    
    InitDialog();    
    
    DXUTCreateWindow( g_wcTitle ); 
    DXUTCreateDevice( true, 1200, 540 );  

    DXUTMainLoop(); // Enter into the DXUT render loop

    // Perform any application-level cleanup here
    return DXUTGetExitCode();
}
